/*
BOIL (Brunni's Own Interpreter Language)
(c) 1998 netEstate (http://www.netestate.de/, info@netestate.de),
Michael Brunnbauer & Markus Hendel, Gesellschaft fuer Netzwerkkommunikation bR
Am Bluetenanger 8, 80995 Muenchen, Germany

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "general.h"
#ifdef WITH_RPC
#include "rpc.h"
#endif
#include "handles.cc"
#include "boil.h"
#include "boil.tab.h"
#include "libs.h"

int args_inet_aton[]= { TYPE_VALUE_STRING,
			0 };

flowctrl func_inet_aton(type_block* aktblock,opt* options,
			joined_list<type_value>* args)
{
  aktblock=aktblock;options=options;args=args; /* to prevent warnings */
  flowctrl back;
  back.ctrl=FLOW_RETURN;
  back.returnval.type=TYPE_VALUE_ULONG;
  back.returnval.val.ulongval=0;

  struct in_addr tmp;

  if (!defined((*args)[0].stringval))
    {
      setvarulong("error",1);
      setvarstring("perror","inet_aton: string not defined");
      return back;
    };
  
  if (!inet_aton((*args)[0].stringval,&tmp))
    {
      setvarulong("error",1);
      setvarstring("perror","inet_aton: inet_aton() failed");
      return back;
    };

  back.returnval.val.ulongval=(unsigned long) tmp.s_addr;
  return back;
}

int args_inet_ntoa[]= { TYPE_VALUE_ULONG,
                        0 };

flowctrl func_inet_ntoa(type_block* aktblock,opt* options,
                        joined_list<type_value>* args)
{
  aktblock=aktblock;options=options;args=args; /* to prevent warnings */
  flowctrl back;
  back.ctrl=FLOW_RETURN;
  back.returnval.type=TYPE_VALUE_STRING;

  struct in_addr tmp;
  tmp.s_addr=(*args)[0].val.ulongval;
  back.returnval.stringval=inet_ntoa(tmp);
  return back;
}

int args_makesock[]= { TYPE_VALUE_STRING,
                       TYPE_VALUE_ULONG,
                       TYPE_VALUE_LONG,
                       TYPE_VALUE_LONG,
                       0 };

flowctrl func_makesock(type_block* aktblock,opt* options,
                       joined_list<type_value>* args)
{
  aktblock=aktblock;options=options;args=args; /* to prevent warnings */
  flowctrl back;
  back.ctrl=FLOW_RETURN;
  back.returnval.type=TYPE_VALUE_LONG;
  back.returnval.val.longval=-1;
  int on=1;

  int sock;
  struct sockaddr_in my_addr;
  struct in_addr tmp;

  if (!defined((*args)[0].stringval))
    {
      setvarulong("error",1);
      setvarstring("perror","makesock: string not defined");
      return back;
    };

  if ((*args)[0].stringval=="")
    {
      tmp.s_addr=htonl(INADDR_ANY);
    }
  else
    {
      if (!inet_aton((*args)[0].stringval,&tmp))
        {
          setvarulong("error",1);
          setvarstring("perror","makesock: syntax error in ip-address");
          return back;
        };
    };
      
  memset((void*)& my_addr,0,sizeof(my_addr));
  my_addr.sin_port=htons((unsigned short int) (*args)[1].val.ulongval);
  my_addr.sin_family=AF_INET;
  my_addr.sin_addr=tmp;

  sock=socket(AF_INET,(int) (*args)[2].val.longval,
              (int) (*args)[3].val.longval);
  if (sock==-1)
    {
      setvarulong("error",1);
      setvarstring("perror","makesock: socket() failed");
      return back;
    };
  if ((*args)[1].val.ulongval)
    {
      if (setsockopt(sock,SOL_SOCKET,SO_REUSEADDR,(void*) &on,sizeof(on)))
	{
	  setvarulong("error",1);
	  setvarstring("perror","makesock: setsockopt() failed");
	  return back;
	};
      if (bind(sock,(struct sockaddr*)&my_addr,sizeof(my_addr))==-1)
	{
	  close(sock);
	  setvarulong("error",1);
	  setvarstring("perror","makesock: bind() failed");
	  return back;
	};
    };
  back.returnval.val.longval=options->ptr_h->sockets.insert(sock);
  return back;
}

int args_closesock[] = { TYPE_VALUE_LONG,
                         0 };

flowctrl func_closesock(type_block* aktblock,opt* options,
                        joined_list<type_value>* args)
{
  aktblock=aktblock;options=options;args=args; /* to prevent warnings */
  flowctrl back;
  back.ctrl=FLOW_RETURN;
  back.returnval.type=TYPE_VALUE_ULONG;
  back.returnval.val.ulongval=0;

  int sock;

  if (!options->ptr_h->sockets.ishandle((*args)[0].val.longval))
    {
      setvarulong("error",1);
      setvarstring("perror","closesock: illegal handle");
      return back;
    };
  
  sock=options->ptr_h->sockets.gethandle((*args)[0].val.longval);
  
  if (!close(sock)) { back.returnval.val.ulongval=1; };
  options->ptr_h->sockets.del((*args)[0].val.longval);
  return back;
}

int args_connectsock[]= { TYPE_VALUE_LONG,
			  TYPE_VALUE_STRING,
			  TYPE_VALUE_ULONG,
			  0 };

flowctrl func_connectsock(type_block* aktblock,opt* options,
			  joined_list<type_value>* args)
{
  aktblock=aktblock;options=options;args=args; /* to prevent warnings */
  flowctrl back;
  back.ctrl=FLOW_RETURN;
  back.returnval.type=TYPE_VALUE_ULONG;
  back.returnval.val.longval=0;

  if (!defined((*args)[1].stringval))
    {
      setvarulong("error",1);
      setvarstring("perror","connectsock: string not defined");
      return back;
    };
  int sock;
  if (!options->ptr_h->sockets.ishandle((*args)[0].val.longval))
    {
      setvarulong("error",1);
      setvarstring("perror","connectsock: illegal handle");
      return back;
    };
  sock=options->ptr_h->sockets.gethandle((*args)[0].val.longval);

  struct in_addr tmp;
  string host=(*args)[1].stringval;
  hostent* h=gethostbyname(host);
  if (h)
    {
      tmp.s_addr=*((unsigned long*) h->h_addr_list[0]);
      host=inet_ntoa(tmp);
    };
  if (!inet_aton(host,&tmp))
    {
      setvarulong("error",1);
      setvarstring("perror","connectsock: destination address not valid");
      return back;
    };
  struct sockaddr_in other_addr;
  memset((void*)& other_addr,0,sizeof(other_addr));
  other_addr.sin_port=htons((int) (*args)[2].val.ulongval);
  other_addr.sin_family=AF_INET;
  other_addr.sin_addr=tmp;

  if (connect(sock,(struct sockaddr*)&other_addr,sizeof(other_addr))==-1)
    {
      setvarulong("error",1);
      setvarstring("perror","connectsock: connect() failed");
      return back;
    };

  back.returnval.val.ulongval=1;
  return back;
}

int args_send[]= { TYPE_VALUE_LONG,
		   TYPE_VALUE_STRING,
		   0 };

flowctrl func_send(type_block* aktblock,opt* options,
		   joined_list<type_value>* args)
{
  aktblock=aktblock;options=options;args=args; /* to prevent warnings */
  flowctrl back;
  back.ctrl=FLOW_RETURN;
  back.returnval.type=TYPE_VALUE_ULONG;
  back.returnval.val.ulongval=0;

  if (!defined((*args)[1].stringval))
    {
      setvarulong("error",1);
      setvarstring("perror","send: string not defined");
      return back;
    };

  int sock;
  if (!options->ptr_h->sockets.ishandle((*args)[0].val.longval))
    {
      setvarulong("error",1);
      setvarstring("perror","send: illegal handle");
      return back;
    };
  sock=options->ptr_h->sockets.gethandle((*args)[0].val.longval);

  if (send(sock,(*args)[1].stringval,(*args)[1].stringval.len,0)==-1)
    {
      setvarulong("error",1);
      setvarstring("perror","send: send() failed");
      return back;
    };

  back.returnval.val.ulongval=1;
  return back;
}  

int args_recv[]= { TYPE_VALUE_LONG,
                   0 };

flowctrl func_recv(type_block* aktblock,opt* options,
                   joined_list<type_value>* args)
{
  aktblock=aktblock;options=options;args=args; /* to prevent warnings */
  flowctrl back;
  back.ctrl=FLOW_RETURN;
  back.returnval.type=TYPE_VALUE_STRING;
  
  int sock;
  if (!options->ptr_h->sockets.ishandle((*args)[0].val.longval))
    {
      setvarulong("error",1);
      setvarstring("perror","recv: illegal handle");
      return back;
    };
  sock=options->ptr_h->sockets.gethandle((*args)[0].val.longval);

  string rueck="";
  char buf[2];
  buf[1]=0;
  int status;

  while ((status=recv(sock,buf,1,0))==1)
    {
      rueck+=buf;
      if (buf[0]=='\n')
	{
	  back.returnval.stringval=rueck;
	  return back;
	};
    };

  if (status==-1)
    {
      setvarulong("error",1);
      setvarstring("perror","recv: recv() failed");
      return back;
    }
  else if (rueck.len)
    {
      back.returnval.stringval=rueck;
    };
  return back;
}


